unit FensterVerwaltung;

interface
uses Windows,Opengl12,sysutils;
//const
// ColorDepth = 32;//Farbtiefe
// WND_TITLE = 'openGL-Fenster';
type
 TGLWindow=class
 private
   FHandle  : HWND;                     // Handle aufs Fenster
   FDC   : HDC;                      // Device Context
   FRC   : HGLRC;                    // OpenGL Rendering Context
   FWidth : Integer;
   FHeight :Integer;
   FFullscreen:Boolean;
   procedure CreateWnd;
   procedure KillWnd;
 protected
 public
   property Handle:HWND read FHandle;
   property  DC: HDC read FDC;
   property  Fullscreen:Boolean read FFullscreen;
   constructor Create(const Width, Height : Integer;const Fullscreen : Boolean);
   destructor Destroy; override;

 end;

implementation
uses MessageControlUnit,//Die Unit welche die Funktion WndProc enthlt.
     GlobaleVariablen;//Enthlt die 2 Konstanten WND_TITLE und ColorDepth
Constructor TGLWindow.Create(const Width, Height : Integer;const Fullscreen : Boolean);
begin
 inherited Create;
 FWidth:= Width;
 FHeight := Height;
 FFullscreen := Fullscreen;
 CreateWnd;
 //raise Exception.Create('Exceptions: '+ #13#10 + Exceptions);
end;

destructor TGLWindow.Destroy;
begin
 KillWnd;
 inherited;
end;

procedure TGLWindow.CreateWnd;
var
  wndClass : TWndClass;         // Fenster Klasse
  dwStyle : DWORD;              // Fenster Stil
  dwExStyle : DWORD;            // Erweiterter Fenster Stil
  dmScreenSettings : DEVMODE;   // Bildschirm Einstellungen (fullscreen, etc...)
  PixelFormat : TGLuint;         // OpenGL Einstellungen (Pixelformat)
  h_Instance : HINST;           // aktuelle Instanz
  pfd : TPIXELFORMATDESCRIPTOR;  // Einstellungen fr das OpenGL Fenster
begin
  h_Instance := GetModuleHandle(nil);       // Instanz fr's Fenster holen
  ZeroMemory(@wndClass, SizeOf(wndClass));  // Daten in wndClass lschen

  with wndClass do                    // Setup der Fenster Klasse
  begin
    style         := CS_HREDRAW or    // Neuzeichenen wenn Fenster-Breite gendert
                     CS_VREDRAW or    // Neuzeichenen wenn Fenster-Hhe gendert
                     CS_OWNDC;        // Device Context exlusiv
    lpfnWndProc   := @WndProc;        // WndProc wird als Window Procedure gesetzt
    hInstance     := h_Instance;
    hCursor       := LoadCursor(0, IDC_ARROW);
    lpszClassName := 'OpenGL';
  end;

  if (RegisterClass(wndClass) = 0) then  // Fenster Klasse registrieren
  begin
    Exception.Create('Failed to register the window class!');
    Exit
  end;

  // Vollbild-Modus
  if FFullscreen then
  begin
    ZeroMemory(@dmScreenSettings, SizeOf(dmScreenSettings));
    with dmScreenSettings do begin              // Bildschirm Einstellungen werden festgelegt
      dmSize       := SizeOf(dmScreenSettings);
      dmPelsWidth  := FWidth;                    // Fenster Breite
      dmPelsHeight := FHeight;                   // Fenster Hhe
      dmBitsPerPel := ColorDepth;               // Farbtiefe (32bit etc)
      dmFields     := DM_PELSWIDTH or DM_PELSHEIGHT or DM_BITSPERPEL;
    end;

    // Bilschirm Modus auf Vollbild setzen
    if (ChangeDisplaySettings(dmScreenSettings, CDS_FULLSCREEN) = DISP_CHANGE_FAILED) then
    begin
      MessageBox(0, 'Unable to switch to fullscreen!', 'Error', MB_OK or MB_ICONERROR);
      FFullscreen := False;
    end;
  end;

  // Wenn wir immer noch im Vollbildmodus sind....
  if (FFullscreen) then
  begin
    dwStyle := WS_POPUP or                // Popup Fenster Stil
               WS_CLIPCHILDREN            // Kein Zeichnen in Child Fenstern
               or WS_CLIPSIBLINGS;        // Kein Zeichnen in Sibling Fenstern
    dwExStyle := WS_EX_APPWINDOW;         // Fenster im Vordergrund

  end
  else // Fr Normale Fenster
  begin
    dwStyle := WS_OVERLAPPEDWINDOW or     // berschneidung zulassen
               WS_CLIPCHILDREN or         // Kein Zeichnen in Child Fenstern
               WS_CLIPSIBLINGS;           // Kein Zeichnen in Sibling Fenstern
    dwExStyle := WS_EX_APPWINDOW or       // Fenster im Fordergrund
                 WS_EX_WINDOWEDGE;        // Erhobener Rand
  end;

  // Das oben definierte Fenster wird erstellt
  FHandle := 0;
  FHandle := CreateWindowEx(dwExStyle,      // Erweiterter Fenster Stil
                          'OpenGL',       // Name der Klasse
                          WND_TITLE,      // Fenster Titel (caption)
                          dwStyle,        // Fenster Stil
                          0, 0,           // Fenster Position
                          FWidth, FHeight,  // Gre des Fensters
                          0,              // Keine Paren-Windows
                          0,              // Kein Men
                          h_Instance,     // die Instanz
                          nil);           // Kein Parameter fr WM_CREATE
  if FHandle = 0 then
  begin
    KillWnd;
    Exception.Create('Unable to create window!');
    Exit;
  end;

  // Den Device Kontext unseres Fensters besorgen
  FDC := GetDC(FHandle);
  if (FDC = 0) then
  begin
    KillWnd;
    Exception.Create('Unable to get a device context!');
    Exit;
  end;

  // Das Pixelformat einstellen
  with pfd do
  begin
    nSize           := SizeOf(TPIXELFORMATDESCRIPTOR); // Gre des Pixel Format Descriptor
    nVersion        := 1;                    // Version des Daten Structs
    dwFlags         := PFD_DRAW_TO_WINDOW    // Buffer erlaubt zeichenen auf Fenster
                       or PFD_SUPPORT_OPENGL // Buffer untersttzt OpenGL drawing
                       or PFD_DOUBLEBUFFER;  // Double Buffering benutzen
    iPixelType      := PFD_TYPE_RGBA;        // RGBA Farbformat
    cColorBits      := ColorDepth;           // OpenGL Farbtiefe
    cRedBits        := 0;
    cRedShift       := 0;
    cGreenBits      := 0;
    cGreenShift     := 0;
    cBlueBits       := 0;
    cBlueShift      := 0;
    cAlphaBits      := 0;                    // Not supported
    cAlphaShift     := 0;                    // Not supported
    cAccumBits      := 0;                    // Kein Accumulation Buffer
    cAccumRedBits   := 0;
    cAccumGreenBits := 0;
    cAccumBlueBits  := 0;
    cAccumAlphaBits := 0;
    cDepthBits      := 16;                   // Genauigkeit des Depht-Buffers
    cStencilBits    := 0;                    // Stencil Buffer ausschalten
    cAuxBuffers     := 0;                    // Not supported
    iLayerType      := PFD_MAIN_PLANE;       // Wird Ignoriert!
    bReserved       := 0;                    // Anzahl der Overlay und Underlay Planes
    dwLayerMask     := 0;                    // Wird Ignoriert!
    dwVisibleMask   := 0;                    // Transparente Farbe der Underlay Plane
    dwDamageMask    := 0;                    // Wird Ignoriert!
  end;

  // Gibt ein untersttzes Pixelformat zurck das dem geforderten so gut wie mglich enspricht
  PixelFormat := ChoosePixelFormat(FDC, @pfd);
  if (PixelFormat = 0) then
  begin
    KillWnd;
    Exception.Create('Unable to find a suitable pixel format');
    Exit;
  end;

  // Das Pixelformat unseres Device Kontexts wird durch das neue ersetzt
  if (not SetPixelFormat(FDC, PixelFormat, @pfd)) then
  begin
    KillWnd;
    Exception.Create('Unable to set the pixel format');
    Exit;
  end;

  // OpenGL Rendering Context wird erstellt
  FRC := wglCreateContext(FDC);
  if (FRC = 0) then
  begin
    KillWnd;
    Exception.Create('Unable to create an OpenGL rendering context');
    Exit;
  end;

  // Der OpenGL Rendering Context wird aktiviert
  if (not wglMakeCurrent(FDC, FRC)) then
  begin
    KillWnd;
    Exception.Create('Unable to activate OpenGL rendering context');
    Exit;
  end;
  (*
  // Initialisierung des Timers zur FPS-Berechnung
  SetTimer(FHandle, FPS_TIMER, FPS_INTERVAL, nil);

  // Initialisierung des Timers zur Allgemeinen-Berechnung
  SetTimer(FHandle, STANDART_TIMER, STANDART_INTERVAL, nil);
  *)

  // Das Fenster wird in Vordergrund gebracht
  ShowWindow(FHandle, SW_SHOW);
  SetForegroundWindow(FHandle);
  SetFocus(FHandle);
  (*
  // Das Fenster bekommt nochmal die Gre zugewiesen um OpenGl richtig zu initialisieren
  if FFullscreen then glResizeWnd(FWidth, FHeight); //Stimmt scheinbar nicht. Macht man dies so erhlt das Fenster eine Gre die es nicht hat.
  glInit(); *)
end;
procedure TglWindow.KillWnd;
begin
  if FFullscreen then begin            // Wenn Vollbild in Standardauflsung zurckkehren
    ChangeDisplaySettings(devmode(nil^), 0);
    ShowCursor(True);
  end;

  // Freigabe des Device und Rendering Contexts.
  ActivateRenderingContext(FDC,FRC);

  // Lscht Rendering Context
  if (not wglDeleteContext(FRC)) then begin
    MessageBox(0, 'Release of rendering context failed!', 'Error', MB_OK or MB_ICONERROR);
    FRC := 0;
  end;

  // Gibt Device Context fre
  if ((FDC > 0) and (ReleaseDC(FHandle, FDC) = 0)) then  begin
    MessageBox(0, 'Release of device context failed!', 'Error', MB_OK or MB_ICONERROR);
    FDC := 0;
  end;

  // Schliet das Fenster
  if ((FHandle <> 0) and (not DestroyWindow(FHandle))) then begin
    MessageBox(0, 'Unable to destroy window!', 'Error', MB_OK or MB_ICONERROR);
    FHandle := 0;
  end;

  // Entfernt window class Registrierung
  if (not UnRegisterClass('OpenGL', hInstance)) then begin
    MessageBox(0, 'Unable to unregister window class!', 'Error', MB_OK or MB_ICONERROR);
    hInstance := 0;
  end;
end;

end.
